==> mobile_pos-0.7.0.tar.gz: lib/src/views/card_payment/connect_bluetooth.dart <==
import 'dart:async';
import 'dart:developer';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:gap/gap.dart';
import 'package:mobile_pos/mobile_pos_platform_interface.dart';
import 'package:mobile_pos/src/helpers/global_variables.dart';
import 'package:mobile_pos/src/helpers/helper_functions.dart';
import 'package:mobile_pos/src/helpers/logger.dart';
import 'package:mobile_pos/src/models/bluetooth_model.dart';
import 'package:mobile_pos/src/models/raven_const.dart';
import 'package:mobile_pos/src/network/api_requests.dart';
import 'package:mobile_pos/src/shared_widgets/progress_dialog.dart';
import 'package:mobile_pos/src/shared_widgets/ravenpay_button_loader.dart';
import 'package:mobile_pos/src/styles/ravenpay_app_colors.dart';
import 'package:mobile_pos/src/styles/ravenpay_textstyles.dart';
import 'package:mobile_pos/src/views/card_payment/insert_card.dart';
import 'package:mobile_pos/src/views/card_payment/widget/see_how_to_connect.dart';
import 'package:mobile_pos/src/shared_widgets/ravenpay_button.dart';
import 'package:mobile_pos/src/shared_widgets/ravenpay_card.dart';
import 'package:mobile_pos/src/shared_widgets/ravenpay_close_button.dart';
import 'package:mobile_pos/src/shared_widgets/ravenpay_scaffold.dart';
import 'package:ripple_wave/ripple_wave.dart';

class ConnectBluetooth extends StatefulWidget {
  final double amount;
  final PaymentType paymentType;
  const ConnectBluetooth(
      {super.key, required this.amount, this.paymentType = PaymentType.card});

  @override
  State<ConnectBluetooth> createState() => _ConnectBluetoothState();
}

class _ConnectBluetoothState extends State<ConnectBluetooth>
    with SingleTickerProviderStateMixin {
  late AnimationController controller;
  StreamSubscription? streamSubscription;
  List<BluetoothModel> availableDevies = [];
  BluetoothModel? selectedDevice;
  bool isLoading = false;
  bool scanning = true;
  bool isSettingBankBox = false;
  bool? status;
  @override
  void initState() {
    controller = AnimationController(
      duration: const Duration(seconds: 3),
      vsync: this,
    );
    selectedDevice = null;
    isSettingBankBox = false;
    super.initState();
    streamSubscription = MobilePosPlatform
        .instance.channelStreamController!.stream
        .listen((event) {
      if (event.eventType == RavenMPosEventType.searchedDevices) {
        controller.stop();
        verifyDevice(event.data);
        setState(() {});
      }
      if (event.eventType == RavenMPosEventType.scanning) {
        controller.repeat();
        scanning = true;
        setState(() {});
      }

      if (event.eventType == RavenMPosEventType.stopScanning) {
        controller.stop();
        scanning = false;
        setState(() {});
      }
      if (event.eventType == RavenMPosEventType.connectionErr) {
        controller.stop();
        isLoading = false;
        setState(() {});
        showSnack(context, "unable to connect device... please try again");
        setState(() {});
      }
      if (event.eventType == RavenMPosEventType.deviceNotDetectedAnymore) {
        controller.stop();
        isLoading = false;
        setState(() {});
        availableDevies = [];
        showSnack(
            context, "can't find/locate bankbox anymore... please rescan");
        setState(() {});
      }
      if (event.eventType == RavenMPosEventType.settingBankBox) {
        controller.stop();
        isSettingBankBox = true;
        isLoading = false;
        setState(() {});
        showDialog(
            barrierDismissible: false,
            context: context,
            builder: (BuildContext context) => const ProgressDialog(
                  status: 'Setting up Bankbox...',
                ));
      }
      if (event.eventType == RavenMPosEventType.settingCompleted) {
        controller.stop();
        updateBankBoxInjection();
      }
      if (event.eventType == RavenMPosEventType.connected) {
        logData("device connected successfully...");
        setState(() {});
      }

      if (event.eventType == RavenMPosEventType.card) {
        if (isSettingBankBox) {
          Navigator.pop(context);
        }
        replaceRoute(
            context,
            InsertCard(
              paymentType: widget.paymentType,
              deviceName: availableDevies[currentIndex].title.toUpperCase(),
              amount: widget.amount,
            ));
        streamSubscription!.cancel();
      }
    });
  }

  void verifyDevice(BluetoothModel device) async {
    if (!device.title.toUpperCase().contains("MPOS")) {
      log("❗️❗️❗️DEVICCE REJECTED ${device.title}");
      return;
    }
    var response =
        await ApiRequests.getBluetoothDeviceInfo(serialNumber: device.serialNo);
    if (response != false) {
      device.name = response['business_name'];
      device.title = response['tid_label'];
      device.cardInjection = response['card_injection'];
      bool hasDevice =
          availableDevies.any((element) => element.serialNo == device.serialNo);
      if (!hasDevice) {
        availableDevies.add(device);
      }
      setState(() {});
    }
  }

  void updateBankBoxInjection() async {
    if (selectedDevice == null) {
      return;
    }
    ApiRequests.updateBankBoxInjection(serialNumber: selectedDevice!.serialNo);
  }

  int currentIndex = 0;
  @override
  Widget build(BuildContext context) {
    Size size = MediaQuery.of(context).size;
    return WillPopScope(
      onWillPop: () async {
        return false;
      },
      child: RavenPayScaffold(
          showLogo: true,
          body: Padding(
            padding: EdgeInsets.symmetric(horizontal: kHoriontalScreenPadding),
            child: Column(
              children: [
                const Gap(24),
                Row(
                  children: [
                    RavenPayCloseButton(
                      isArrow: true,
                      text: "",
                      onBackPressed: () {
                        if (!isLoading) {
                          streamSubscription!.cancel();
                          streamSubscription = null;
                          MobilePosPlatform.instance.disconnectDevice();
                          Navigator.pop(context);
                        } else {
                          showSnack(context, "Please wait... still connecting");
                        }
                      },
                    ),
                    const Spacer(),
                    const SeeHowToConnect()
                  ],
                ),
                Gap(size.height * 0.08),
                CirculBluetoothImage(
                  controller: controller,
                ),
                const Gap(16),
                AnimatedOpacity(
                  opacity: scanning ? 1 : 0,
                  duration: const Duration(seconds: 1),
                  child: Text(
                    "Searching for device...",
                    style: subtitle.copyWith(color: pluginTheme.primaryColor),
                  ),
                ),
                const Spacer(),
                Padding(
                  padding: const EdgeInsets.symmetric(horizontal: 4),
                  child: Row(
                    children: [
                      Expanded(
                        child: Text("Available Connections",
                            style: subtitle2.copyWith(
                                color: pluginTheme.primaryColor)),
                      ),
                      AnimatedOpacity(
                        opacity: scanning ? 0 : 1,
                        duration: const Duration(milliseconds: 500),
                        child: InkWell(
                          splashColor: Colors.transparent,
                          focusColor: Colors.transparent,
                          hoverColor: Colors.transparent,
                          highlightColor: Colors.transparent,
                          onTap: () {
                            if (isLoading) {
                              return;
                            }
                            controller.repeat();
                            availableDevies = [];
                            MobilePosPlatform.instance.scanForMPosDevice();
                          },
                          child: Row(
                            children: [
                              Text("Refresh",
                                  style: subtitle2.copyWith(
                                      color: pluginTheme.primaryColor)),
                              Icon(
                                Icons.refresh_rounded,
                                size: 20,
                                color: pluginTheme.primaryColor,
                              )
                            ],
                          ),
                        ),
                      )
                    ],
                  ),
                ),
                const Gap(8),
                RavenPayCard(
                    padding: EdgeInsets.zero,
                    height: 180,
                    child: availableDevies.isEmpty
                        ? Center(
                            child: scanning
                                ? RavenButtonLoader(
                                    color: pluginTheme.primaryColor,
                                    size: 30,
                                  )
                                : Text("No Device Found Yet",
                                    style: subtitle.copyWith(fontSize: 12)))
                        : ListView.builder(
                            itemCount: availableDevies.length,
                            itemBuilder: (context, index) {
                              return ItemView(
                                  title: availableDevies[index]
                                      .title
                                      .toUpperCase(),
                                  currentIndex: currentIndex,
                                  name: availableDevies[index]
                                      .name!
                                      .toUpperCase(),
                                  index: index,
                                  onPressed: (val) {
                                    currentIndex = val;
                                    setState(() {});
                                  });
                            })),
                const Gap(34),
                RavenPayButton(
                  enabled: availableDevies.isNotEmpty,
                  buttonText: "Connect to device",
                  busy: isLoading,
                  onPressed: () {
                    if (availableDevies.isEmpty) {
                      return;
                    }
                    processConnection();
                  },
                ),
                if (pluginConfig.showLabel) ...[
                  const Gap(64),
                ] else ...[
                  const Gap(24),
                ]
              ],
            ),
          )),
    );
  }

  processConnection() {
    showCupertinoDialog(
        context: context,
        builder: (context) => CupertinoAlertDialog(
              content: Text("Proceed to connect with this device ?",
                  style: subtitle.copyWith(fontSize: 12, color: Colors.black)),
              title: Padding(
                padding: const EdgeInsets.only(bottom: 12),
                child: Text(availableDevies[currentIndex].title.toUpperCase(),
                    style: subtitle.copyWith(color: Colors.black)),
              ),
              actions: [
                CupertinoDialogAction(
                  onPressed: () {
                    Navigator.pop(context);
                  },
                  child: const Text("No"),
                ),
                CupertinoDialogAction(
                  child: const Text("Yes"),
                  onPressed: () {
                    isLoading = true;
                    setState(() {});
                    selectedDevice = availableDevies[currentIndex];
                    MobilePosPlatform.instance
                        .connectBluetooth(availableDevies[currentIndex]);

                    Navigator.pop(context);
                  },
                )
              ],
            ));
  }
}

class CirculBluetoothImage extends StatelessWidget {
  final AnimationController controller;
  const CirculBluetoothImage({super.key, required this.controller});

  @override
  Widget build(BuildContext context) {
    return SizedBox(
        height: 130,
        width: 130,
        child: RippleWave(
            animationController: controller,
            color: pluginTheme.primaryColor.withOpacity(0.2),
            childTween: Tween(begin: 0.2, end: 0.25),
            child: Image.asset(loadAsset("bluetooth_icon.png"))));
  }
}

class ItemView extends StatelessWidget {
  final String name;
  final String title;
  final int index;
  final int currentIndex;
  final Function(int) onPressed;
  const ItemView({
    super.key,
    required this.index,
    required this.currentIndex,
    required this.title,
    required this.onPressed,
    required this.name,
  });

  @override
  Widget build(BuildContext context) {
    return InkWell(
      splashColor: Colors.transparent,
      highlightColor: Colors.transparent,
      onTap: () => onPressed(index),
      child: Container(
        padding: const EdgeInsets.only(bottom: 8, top: 4),
        decoration: BoxDecoration(
            border: Border(
                bottom: BorderSide(
                    color: AppColors.ravenPayGrey2.withOpacity(0.5)))),
        child: Column(
          children: [
            Row(
              children: [
                Radio(
                    activeColor: pluginTheme.primaryColor,
                    value: currentIndex,
                    groupValue: index,
                    onChanged: (val) {}),
                RavenPayText.bodyText(
                  getName(),
                  fontSize: 14,
                )
              ],
            ),
          ],
        ),
      ),
    );
  }

  String getName() {
    String businessName = name;
    String posTitle = title;
    if (businessName.length > 15) {
      businessName = businessName.substring(0, 14);
    }
    if (posTitle.length > 10) {
      posTitle = posTitle.substring(0, 9);
    }
    pluginConfig.appInfo.deviceName = posTitle;
    return "$businessName - $posTitle";
  }
}

==> mobile_pos-0.8.18.tar.gz: lib/src/views/card_payment/connect_bluetooth.dart <==
import 'dart:async';
import 'dart:developer';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:gap/gap.dart';
import 'package:mobile_pos/mobile_pos_platform_interface.dart';
import 'package:mobile_pos/src/helpers/global_variables.dart';
import 'package:mobile_pos/src/helpers/helper_functions.dart';
import 'package:mobile_pos/src/helpers/logger.dart';
import 'package:mobile_pos/src/models/bluetooth_model.dart';
import 'package:mobile_pos/src/models/raven_const.dart';
import 'package:mobile_pos/src/network/api_requests.dart';
import 'package:mobile_pos/src/shared_widgets/progress_dialog.dart';
import 'package:mobile_pos/src/shared_widgets/ravenpay_button_loader.dart';
import 'package:mobile_pos/src/styles/ravenpay_app_colors.dart';
import 'package:mobile_pos/src/styles/ravenpay_textstyles.dart';
import 'package:mobile_pos/src/views/card_payment/insert_card.dart';
import 'package:mobile_pos/src/views/card_payment/widget/see_how_to_connect.dart';
import 'package:mobile_pos/src/shared_widgets/ravenpay_button.dart';
import 'package:mobile_pos/src/shared_widgets/ravenpay_card.dart';
import 'package:mobile_pos/src/shared_widgets/ravenpay_close_button.dart';
import 'package:mobile_pos/src/shared_widgets/ravenpay_scaffold.dart';
import 'package:ripple_wave/ripple_wave.dart';

class ConnectBluetooth extends StatefulWidget {
  final double amount;
  final PaymentType paymentType;
  const ConnectBluetooth(
      {super.key, required this.amount, this.paymentType = PaymentType.card});

  @override
  State<ConnectBluetooth> createState() => _ConnectBluetoothState();
}

class _ConnectBluetoothState extends State<ConnectBluetooth>
    with SingleTickerProviderStateMixin {
  late AnimationController controller;
  StreamSubscription? streamSubscription;
  List<BluetoothModel> availableDevies = [];
  BluetoothModel? selectedDevice;
  bool isLoading = false;
  bool scanning = true;
  bool isSettingBankBox = false;
  bool? status;
  @override
  void initState() {
    controller = AnimationController(
      duration: const Duration(seconds: 3),
      vsync: this,
    );
    selectedDevice = null;
    isSettingBankBox = false;
    super.initState();
    streamSubscription = MobilePosPlatform
        .instance.channelStreamController!.stream
        .listen((event) {
      if (event.eventType == RavenMPosEventType.searchedDevices) {
        controller.stop();
        verifyDevice(event.data);
        setState(() {});
      }
      if (event.eventType == RavenMPosEventType.scanning) {
        controller.repeat();
        scanning = true;
        setState(() {});
      }

      if (event.eventType == RavenMPosEventType.stopScanning) {
        controller.stop();
        scanning = false;
        setState(() {});
      }
      if (event.eventType == RavenMPosEventType.connectionErr) {
        controller.stop();
        isLoading = false;
        setState(() {});
        showSnack(context, "unable to connect device... please try again");
        setState(() {});
      }
      if (event.eventType == RavenMPosEventType.deviceNotDetectedAnymore) {
        controller.stop();
        isLoading = false;
        setState(() {});
        availableDevies = [];
        showSnack(
            context, "can't find/locate bankbox anymore... please rescan");
        setState(() {});
      }
      if (event.eventType == RavenMPosEventType.settingBankBox) {
        controller.stop();
        isSettingBankBox = true;
        isLoading = false;
        setState(() {});
        showDialog(
            barrierDismissible: false,
            context: context,
            builder: (BuildContext context) => const ProgressDialog(
                  status: 'Setting up Bankbox...',
                ));
      }
      if (event.eventType == RavenMPosEventType.settingCompleted) {
        controller.stop();
        updateBankBoxInjection();
      }
      if (event.eventType == RavenMPosEventType.connected) {
        logData("device connected successfully...");
        setState(() {});
      }

      if (event.eventType == RavenMPosEventType.card) {
        if (isSettingBankBox) {
          Navigator.pop(context);
        }
        replaceRoute(
            context,
            InsertCard(
              paymentType: widget.paymentType,
              deviceName: availableDevies[currentIndex].title.toUpperCase(),
              amount: widget.amount,
            ));
        streamSubscription!.cancel();
      }
    });
  }

  void verifyDevice(BluetoothModel device) async {
    if (!device.title.toUpperCase().contains("MPOS")) {
      log("❗️❗️❗️DEVICCE REJECTED ${device.title}");
      return;
    }
    var response =
        await ApiRequests.getBluetoothDeviceInfo(serialNumber: device.serialNo);
    if (response != false) {
      device.name = response['business_name'];
      device.title = response['tid_label'];
      device.cardInjection = response['card_injection'];
      bool hasDevice =
          availableDevies.any((element) => element.serialNo == device.serialNo);
      if (!hasDevice) {
        availableDevies.add(device);
      }
      setState(() {});
    }
  }

  void updateBankBoxInjection() async {
    if (selectedDevice == null) {
      return;
    }
    ApiRequests.updateBankBoxInjection(serialNumber: selectedDevice!.serialNo);
  }

  int currentIndex = 0;
  @override
  Widget build(BuildContext context) {
    Size size = MediaQuery.of(context).size;
    return WillPopScope(
      onWillPop: () async {
        return false;
      },
      child: RavenPayScaffold(
          showLogo: true,
          body: Padding(
            padding: EdgeInsets.symmetric(horizontal: kHoriontalScreenPadding),
            child: Column(
              children: [
                const Gap(24),
                Row(
                  children: [
                    RavenPayCloseButton(
                      isArrow: true,
                      text: "",
                      onBackPressed: () {
                        if (!isLoading) {
                          streamSubscription!.cancel();
                          streamSubscription = null;
                          MobilePosPlatform.instance.disconnectDevice();
                          Navigator.pop(context);
                        } else {
                          showSnack(context, "Please wait... still connecting");
                        }
                      },
                    ),
                    const Spacer(),
                    const SeeHowToConnect()
                  ],
                ),
                Gap(size.height * 0.08),
                CirculBluetoothImage(
                  controller: controller,
                ),
                const Gap(16),
                AnimatedOpacity(
                  opacity: scanning ? 1 : 0,
                  duration: const Duration(seconds: 1),
                  child: Text(
                    "Searching for device(s)...",
                    style: subtitle.copyWith(color: pluginTheme.primaryColor),
                  ),
                ),
                const Spacer(),
                Padding(
                  padding: const EdgeInsets.symmetric(horizontal: 4),
                  child: Row(
                    children: [
                      Expanded(
                        child: Text("Available Devices",
                            style: subtitle2.copyWith(
                                color: pluginTheme.primaryColor)),
                      ),
                      AnimatedOpacity(
                        opacity: scanning ? 0 : 1,
                        duration: const Duration(milliseconds: 500),
                        child: InkWell(
                          splashColor: Colors.transparent,
                          focusColor: Colors.transparent,
                          hoverColor: Colors.transparent,
                          highlightColor: Colors.transparent,
                          onTap: () {
                            if (isLoading) {
                              return;
                            }
                            controller.repeat();
                            availableDevies = [];
                            MobilePosPlatform.instance.scanForMPosDevice();
                          },
                          child: Row(
                            children: [
                              Text("Refresh",
                                  style: subtitle2.copyWith(
                                      color: pluginTheme.primaryColor)),
                              Icon(
                                Icons.refresh_rounded,
                                size: 20,
                                color: pluginTheme.primaryColor,
                              )
                            ],
                          ),
                        ),
                      )
                    ],
                  ),
                ),
                const Gap(8),
                RavenPayCard(
                    padding: EdgeInsets.zero,
                    height: 180,
                    child: availableDevies.isEmpty
                        ? Center(
                            child: scanning
                                ? RavenButtonLoader(
                                    color: pluginTheme.primaryColor,
                                    size: 18,
                                  )
                                : Text("No device found",
                                    style: subtitle.copyWith(fontSize: 12)))
                        : ListView.builder(
                            itemCount: availableDevies.length,
                            itemBuilder: (context, index) {
                              return ItemView(
                                  title: availableDevies[index]
                                      .title
                                      .toUpperCase(),
                                  currentIndex: currentIndex,
                                  name: availableDevies[index]
                                      .name!
                                      .toUpperCase(),
                                  index: index,
                                  onPressed: (val) {
                                    currentIndex = val;
                                    setState(() {});
                                  });
                            })),
                const Gap(34),
                RavenPayButton(
                  enabled: availableDevies.isNotEmpty,
                  buttonText: "Connect to device",
                  busy: isLoading,
                  onPressed: () {
                    if (availableDevies.isEmpty) {
                      return;
                    }
                    processConnection();
                  },
                ),
                if (pluginConfig.showLabel) ...[
                  const Gap(64),
                ] else ...[
                  const Gap(24),
                ]
              ],
            ),
          )),
    );
  }

  processConnection() {
    showCupertinoDialog(
        context: context,
        builder: (context) => CupertinoAlertDialog(
              content: Text("Proceed to connect with this device ?",
                  style: subtitle.copyWith(fontSize: 12, color: Colors.black)),
              title: Padding(
                padding: const EdgeInsets.only(bottom: 12),
                child: Text(availableDevies[currentIndex].title.toUpperCase(),
                    style: subtitle.copyWith(color: Colors.black)),
              ),
              actions: [
                CupertinoDialogAction(
                  onPressed: () {
                    Navigator.pop(context);
                  },
                  child: const Text("No"),
                ),
                CupertinoDialogAction(
                  child: const Text("Yes"),
                  onPressed: () {
                    isLoading = true;
                    setState(() {});
                    selectedDevice = availableDevies[currentIndex];
                    MobilePosPlatform.instance
                        .connectBluetooth(availableDevies[currentIndex]);

                    Navigator.pop(context);
                  },
                )
              ],
            ));
  }
}

class CirculBluetoothImage extends StatelessWidget {
  final AnimationController controller;
  const CirculBluetoothImage({super.key, required this.controller});

  @override
  Widget build(BuildContext context) {
    return SizedBox(
        height: 130,
        width: 130,
        child: RippleWave(
            animationController: controller,
            color: pluginTheme.primaryColor.withOpacity(0.2),
            childTween: Tween(begin: 0.2, end: 0.25),
            child: Image.asset(loadAsset("bluetooth_icon.png"))));
  }
}

class ItemView extends StatelessWidget {
  final String name;
  final String title;
  final int index;
  final int currentIndex;
  final Function(int) onPressed;
  const ItemView({
    super.key,
    required this.index,
    required this.currentIndex,
    required this.title,
    required this.onPressed,
    required this.name,
  });

  @override
  Widget build(BuildContext context) {
    return InkWell(
      splashColor: Colors.transparent,
      highlightColor: Colors.transparent,
      onTap: () => onPressed(index),
      child: Container(
        padding: const EdgeInsets.only(bottom: 8, top: 4),
        decoration: BoxDecoration(
            border: Border(
                bottom: BorderSide(
                    color: AppColors.ravenPayGrey2.withOpacity(0.5)))),
        child: Column(
          children: [
            Row(
              children: [
                Radio(
                    activeColor: pluginTheme.primaryColor,
                    value: currentIndex,
                    groupValue: index,
                    onChanged: (val) {}),
                RavenPayText.bodyText(
                  getName(),
                  fontSize: 14,
                )
              ],
            ),
          ],
        ),
      ),
    );
  }

  String getName() {
    String businessName = name;
    String posTitle = title;
    if (businessName.length > 15) {
      businessName = businessName.substring(0, 14);
    }
    if (posTitle.length > 10) {
      posTitle = posTitle.substring(0, 9);
    }
    pluginConfig.appInfo.deviceName = posTitle;
    return "$businessName - $posTitle";
  }
}
